import numpy as np
from hw2_utils import TestCase, run_tests

import tensorflow as tf
from tensorflow.keras import Input
from tensorflow.keras.utils import to_categorical


class TestCifar(TestCase):
    def __init__(self, *args, **kwargs):
        self.module_name = "hw2_cifar"
        TestCase.__init__(self, *args, **kwargs)

    def test01_build(self):
        """Test for model construction."""

        X = Input(shape=(32, 32, 3))

        logits, probits = self.solution.build_model(X, 10, is_training=True)

        self.assertIsInstance(logits, tf.Tensor)
        self.assertIsInstance(probits, tf.Tensor)

    def test02_run(self):
        """Test a train steps on random data."""

        # Seed is for reproduction purpose. Do not change it.
        np.random.seed(4096)
        tf.compat.v1.random.set_random_seed(1234)

        fake_data = np.random.randn(10, 32, 32, 3)
        fake_label = to_categorical(np.zeros((10)) + 3, 10)

        X = Input(shape=(32, 32, 3))
        Y = tf.placeholder(tf.float32, shape=(None, 10))
        logit, probit = self.solution.build_model(X, 10, is_training=True)
        mean_loss, optimizer = self.solution.build_loss_optimizer(Y, logit)

        # batch normalization in tensorflow requires this extra dependency
        extra_update_ops = tf.get_collection(tf.GraphKeys.UPDATE_OPS)
        with tf.control_dependencies(extra_update_ops):
            train_step = optimizer.minimize(mean_loss)

        with tf.Session() as sess:
            sess.run(tf.global_variables_initializer())
            for _ in range(5):
                _, loss = sess.run(
                    [train_step, mean_loss],
                    feed_dict={X: fake_data, Y: fake_label}
                )
            print("Loss: ", loss)


if __name__ == '__main__':
    run_tests(TestCifar)
